;; File: GRIDTOOL.LSP (C) Ben Olasov  olasov@cs.columbia.edu
;; GridTool compositional grid tool
;; (C) Ben Olasov olasov@cs.columbia.edu 1992,1994
;; GridTool is a design aid for laying out compositional grids
;; with user definable primary and secondary interstitial spaces.
;; *** Note ***   GridTool can be used to draw partially
;; overlapping grids using 'XOR ink' (color -1 in 'grdraw') and
;; the resulting screens can be captured under Windows and SunOs as
;; bitmap files.  Arbitrarily complex layouts can be produced by
;; overlaying different grids with 'XOR ink'.
;;
;; When GRIDTOOL is run twice in 'Video' mode with th
;; same values for all parameters (# of columns, # of rows,
;; primary and secondary interstitial space dimensions along X and Y
;; axes), the second grid will be drawn precisely over the first.
;; The result of doing this is that the original grid will be completely
;; erased, since overlapping lines drawn with XOR ink are self-erasing.
;; 
;; [Options for modifying GridTool:
;; Some unreferenced functions are provided for producing grid using 
;; cell elements other than rectangles (e.g. 'U' , 'n' , etc.).
;; To use these alternate functions, edit the #GEO_MODE1 and #GEO_MODE2
;; assignments in GRID_INIT by changing '_RECT to 'U or 'n.
;; You can also create new cell operations such as diagonal lines, etc.
;; by assigning those operations to #GEO_MODE1 and #GEO_MODE2.]
 
(defun c:gridtool ()
       (grid_init)
       (setq use_default_square?
             (userstr (if use_default_square? use_default_square? "Y")
                      "Use full-screen square to define grid boundary"))
       (if (not (equal (strcase use_default_square?) "Y"))
           (setq #ur (getcorner (setq #ll (getpoint "\nLower left corner: "))
                             "\nUpper right corner: ")))
       (setq #numcols (userint (if #numcols #numcols 3)
                               "\nNumber of columns")
             #numrows (userint (if #numrows #numrows 3)
                              "\nNumber of rows")
             #interstice_h1
             (userdist #ll
                       (if #interstice_h1 #interstice_h1 6.0)
                           "Primary horizontal interstitial space")
             #interstice_h2
             (userdist #ll
                       (if #interstice_h2 #interstice_h2 #interstice_h1)
                       "Secondary horizontal interstitial space")
             #interstice_v1
             userdist #ll
                       (if #interstice_v1 #interstice_v1 #interstice_h1)
                           "Primary vertical interstitial space")
             #interstice_v2
             (userdist #ll
                       (if #interstice_v2 #interstice_v2 #interstice_v1)
                           "Secondary vertical interstitial space"))
       (grid #ll #ur #interstice_h1 #interstice_h2 #interstice_v1 
             #interstice_v2 #numcols #numrows #geo_mode1 #geo_mode2)
       (grid_exit))
       
;; GRID_INIT initializes GRIDTOOL parameters
(defun grid_init ()
       (graphscr)
       (setq primary_mode "A")
       (setq secondary_mode "A")
       (setq display_mode
             (strcase (userstr (if display_mode display_mode "V") 
                 "Draw grids with [P]olylines or grdraw [V]ideo lines [P V]")))
       (setq #geo_mode1 '_rect) ;; change '_RECT to 'U or 'n for variations
       (setq #geo_mode2 '_rect) ;; change '_RECT to 'U or 'n for variations
       (if (not (equal display_mode "V"))
           (listify_layers))
       (setq vctr (getvar "viewctr")
             vsize (getvar "viewsize")
             ~vsize (* vsize 0.8) ; give a little space around the grid
            h~vsize (/ ~vsize 2.0)
             #interstice_h1 (if #interstice_h1
                                #interstice_h1
                                (/ ~vsize 32.0))
             #interstice_h2 (if #interstice_h2
                                #interstice_h2
                                (/ ~vsize 64.0))
             #interstice_v1 (if #interstice_v1
                                #interstice_v1
                                (/ ~vsize 32.0))
             #interstice_v2 (if #interstice_v2
                                #interstice_v2
                                (/ ~vsize 64.0))
             #ur (list (+ (car vctr) h~vsize)
                       (+ (cadr vctr) h~vsize))
             #ll (list (- (car vctr) h~vsize)
                       (- (cadr vctr) h~vsize)))
       (modesave '("cmdecho" blipmode" "attdia"))
       (if (getvar "filedia")
           (setvar "filedia" 0))
       (setvar "attdia" 0)
       (setvar "cmdecho" 0)
       (setvar "expert" 3)
       (setvar "fillmode" 0) 
       (setvar "blipmode" 0)
;       (user_err)
)
 
;; GRID draws a compositional grid with fully definable primary and secondary
;; interstitial paces.  By using the ([V]ideo) mode, complex overlapping
;; grids can be produced by taking advantage of the 'XOR ink' characteristics
;; of GRDRAW color -1.
(defun grid ($ll $ur $interstice_h1 $interstice_h2 $interstice_v1
             $interstice_v2 $numcols $numrows $geo_mode1 $geo_mode2
             / $pts1 $pts2 overall_wid overall_hgt interstice_wid_tot
               net_wid net_hgt unit_wid unit_hgt unit_displacement_h
               unit_displacement_v $col $row)
       ;; assign values to length variables
       (setq overall_wid (abs (- (car $ur) (car $ll)))
             overall_hgt (abs (- (cadr $ur) (cadr $ll)))
             interstice_wid_tot (* $interstice_h1 (1- $numcols))
             interstice_hgt_tot (* $interstice_v1 (1- $numrows))
             net_wid (- overall_wid interstice_wid_tot)
             net_hgt (- overall_hgt interstice_hgt_tot)
             unit_wid (/ net_wid $numcols)
             unit_hgt (/ net_hgt $numrows)
             unit_displacement_h (+ unit_wid $interstice_h1)
             unit_displacement_v (+ unit_hgt $interstice_v1)
             $col 1
             $row 1)
       ;; assign initial values to point variables
       (setq $ll1 $ll
             $ur1 (list (+ (car $ll) unit_wid)
                       (+ (cadr $ll) unit_hgt))
             $pts1 (list $ll1 $ur1))
       (if secondary_mode
           (setq $ll2 (list (+ (car $ll1) $interstice_h2)
                            (+ (cadr $ll1) $interstice_v2))
                 $ur2 (list (- (car $ur1) $interstice_h2)
                            (- (cadr $ur1) $interstice_v2))
                 $pts2 (list $ll2 $ur2)))
       (repeat $numrows
               (setq $col 1)
               (repeat $numcols
                       (cond ((> $col 1)
                              (setq $pts1
                                    (translate_group $pts1
                                                     0.0 
                                                     unit_displacement_h))
                              (if secondary_mode
                                  (setq $pts2
                                        (translate_group $pts2
                                                         0.0 
                                                         unit_displacement_h)))))
                       (if (/= (strcase display_mode) "V")
                               (makelayer "PRIMARY" "5" "CONTINUOUS"))
                       (apply $geo_mode1 $pts1)
                       (if secondary_mode
                           (progn (if (/= (strcase display_mode "V")
                                      (makelayer "SECONDARY" "9" "CONTINUOUS"))
                                  (apply $geo_mode2 $pts2)))
                       (setq $col (1+ $col)))
              (setq $pts1 (translate_group (list $ll1 $ur1)
                                           (/ pi 2.0)
                                           (* unit_displacement_v $row)))
              (if secondary_mode
                 (setq $pts2 (translate_group (list $ll2 $ur2)
                                               (/ pi 2.0)
                                               (* unit_displacement_v $row))))
              (setq $row (1+ $row))))
 
;; GRDRAW_RECT draws a ccw rectangle with grdraw lines.
;;
;;     ul     ur
;;       +<--+
;;       |   ^
;;           |
;;       +-->+
;;     ll     lr
;;
(defun grdraw_rect (&ll &ur col / &lr &ul)
       (setq &lr (list (car &ur) (cadr &ll))
             &ul (list (car &ll) (cadr &ur)))
       (grdraw &ll &lr col)
       (grdraw &lr &ur col)
       (grdraw &ur &ul col)
       (grdraw &ul &ll col))
 
;; TRANSLATE_GROUP returns the list of coordinate triplets
;; @GROUP shifted by @ANGLE degrees and @DIST distance.
;; The effect of using TRANSLATE_GROUP is the same as mapping
;; 'POLAR @ANGLE @DIST over all the points in the group @GROUP.
(defun translate_group (@group @angle @dist / @@group)
       (foreach @point @group
                (setq @point (polar @point @angle @dist))
                (if (null @@group)
                    (setq @@group (list @point))
                    (setq @@group (cons @point @@group))))
       (reverse @@group))
 
;; _RECT draws a rectangle given lower left and upper right
;; corners as arguments
(defun _rect (!ll !ur / !lr !ul)
       (setq !lr (list (car !ur) (cadr !ll))
             !ul (list (car !ll) (cadr !ur)))
       (if (not (equal (strcase display_mode) "V"))
            (command "pline" !ll !ul !ur !lr !ll "")
            (grdraw_rect !ll !ur -1)))
 
;; U draws a "U" shape given lower left and upper right
;; points as arguments
;;   +    +
;;   |    |
;;   +----+
(defun U (!ll !ur / !lr !ul)
       (setq !lr (list (car !ur) (cadr !ll))
             !ul (list (car !ll) (cadr !ur)))
        (if (equal (strcase display_mode) "P")
            (command "pline" !ul !ll !lr !ur "")
            (progn (grdraw_rect !ll !ur -1)
                   (grdraw !ul !ur -1))))
 
;; draw an inverted "U" shape given lower left and upper 
;; right points as arguments
;;   +----+
;;   |    |
;;   +    +
(defun n (!ll !ur / !lr !ul)
       (setq !lr (list (car !ur) (cadr !ll))
             !ul (list (car !ll) (cadr !ur)))
        (if (equal (strcase display_mode) "P")
            (command "pline" !ll !ul !ur !lr "")
            (progn (grdraw_rect !ll !ur -1)
                   (grdraw !ll !lr -1))))
 
;; MAKELAYER creates a new layer with name NAM, color COLOR and linetype LT.
(defun makelayer (nam color lt)
       (if (member nam layers)
           (command "layer" "t" nam "m" nam "c" color nam "lt" LT nam "")
           (command "layer" "m" nam "c" color nam "lt" LT nam "")))
 
;;; LISTIFY_LAYERS returns a list of all layers in drawing in order created
(defun listify_layers (/ layer nam)
       (setq layer (cdr (assoc 2 (tblnext "layer" T))) ;;rewind layer table
             layers (list  layer))
       (while (setq layer (tblnext "layer")) ;;construct layer list
              (setq nam (cdr (assoc 2 layer))
                    layers (cons nam layers))) (reverse layers))
 
;; USERDIST prompts the user for a distance.  The user may respond by
;; either digitizing a distance or entering a number from the keyboard.
;; If a default value is passed to USERDIST, that value will be returned
;; if the user simply hits 'enter'.  USERDIST returns a real number.
(defun userdist (refpt dflt prmpt / var)
       (if (setq var
                 (getdist (if refpt refpt (getvar "insbase"))
                          (if dflt (strcat prmpt " <" (rtos dflt 4 5) ">: ")
                                   (strcat prmpt ": ")))) var dflt))
 
;; USERINT prompts the user for an integer.  The user may enter a number
;; or accept the default by hittin 'enter'.  USERINT returns an integer.
(defun userint (dflt prmpt / var)
       (if setq var
                 (getint (if dflt (strcat prmpt " <" (itoa dflt) ">: ")
                                  (strcat prmpt ": ")))) var dflt))
 
;; USERSTR prompts the user for a string.  The user may enter a string
;; or accept the default by hitting 'enter'.  USERSTR returns a string.
(defun userstr (dflt prmpt / var) ;;DFLT and PRMPT are strings
       (setq var (getstring t (if (and dflt (/= dflt ""))
                                (strcat prmpt " <" dflt ">: ")
                                (strcat prmpt ": "))))
       (cond ((/= var "") var)
             ((and dflt (= var "")) dflt)
             (T "")))
 
(defun grid_exit ()
       (moderestore)
       (setq *error* $err$)
       (princ))
 
(defun user_err ()
      (setq $err$ (eval '*error*))
      (defun *ERROR* (msg)
             (moderestore)
             (princ "*error*: ")
             (princ msg)
             (princ)))
 
;; modified MODES from AutoDesk
;; example of expected syntax: (modes '("cmdecho" "blipmode"))
(defun modesave (a)
       (setq c_lay (getvar "clayer")
             mlst '())
       (repeat (length a)
               (setq mlst (apend mlst (list (list (car a) (getvar (car a))))))
               (setq a (cdr a))))
 
;; modified MODER from AutoDesk
(defun moderestore ()
       (if (and c_lay
                (= (type c_lay) 'STR)  ;;in case something redefined it
                (/= c_lay (getvar "clayer")))
           (command "layer" "s" c_lay ""))
       (repeat (length mlst)
               (setvar (caar mlst) (cadar mlst))
               (setq mlst (cdr mlst))))
 
(princ "\nAutomatically starting GRIDTOOL - type GRIDTOOL to run again.\n")
(c:gridtool)
 

